BEGIN {
	title = "techn0path - no title"
	body = ""
	subHeadings[0] = ""
}

/^[ 	]*$/ { next }

/^#[^#]/ {
	match($0, "^#[ ]*")
	headingText = substr($0, RSTART + RLENGTH)
	title = headingText
	body = body sprintf("<h1>%s</h1>\n", headingText)

	next
}

/^##[^#]/ {
	match($0, "^##[ ]*") 
	headingText = substr($0, RSTART + RLENGTH)
	subHeadings[length(subHeadings)] = headingText

	headingTextAsID = tolower(headingText)
	gsub(/^[ ]*/, "", headingTextAsID)
	gsub(/[ ]*$/, "", headingTextAsID)
	gsub(" ", "_", headingTextAsID)

	body = body sprintf("<h2 id='%s'>%s</h2>\n\
	<a href='#%s'>Permalink</a><br/><br/>\n", \
		headingTextAsID, headingText, headingTextAsID)

	next
}

/^###[^#]/ {
	match($0, "^###[ ]*") 
	headingText = substr($0, RSTART + RLENGTH)

	body = body sprintf("<h3>%s</h3>\n", headingText)

	next
}

/^=>/ {
	match($0, "^=>[ ]*[^ ]*") 
	link = substr($0, 3, RLENGTH - 2)
	description = substr($0, RSTART + RLENGTH)

	sub("^[ ]*", "", link)
	sub("^[ ]*", "", description)

	if (link !~ "^[a-zA-Z0-9_]*:") 
		sub("\.gmi$", ".html", link)

	if (description == "") description = link

	gsub("&", "\\&amp;", description)
	gsub("<", "\\&lt;", description)
	gsub(">", "\\&gt;", description)
	gsub("'", "\\&apos;", description)
	gsub("\"", "\\&quot;", description)

	body = body sprintf("<a href='%s'>%s</a><br>\n", link, description)

	next
}

/^```/ {
	body = body "<pre>\n"

	getline

	while (match($0, "^```") == 0) {
		gsub("&", "\\&amp;")
		gsub("<", "\\&lt;")
		gsub(">", "\\&gt;")
		gsub("'", "\\&apos;")
		gsub("\"", "\\&quot;")
		body = body $0 "\n"
		
		if (getline != 1) break
	}

	body = body "</pre>\n"

	next
}

/^>/ {
	body = body "<blockquote>\n"

	while (match($0, /^>[ ]*/) != 0) {
		text = substr($0, RSTART + RLENGTH)

		gsub("&", "\\&amp;", text)
		gsub("<", "\\&lt;", text)
		gsub(">", "\\&gt;", text)
		gsub("'", "\\&apos;", text)
		gsub("\"", "\\&quot;", text)

		body = body text "<br>\n"
		
		if (getline != 1) break
	}

	body = body "</blockquote>\n"
	next
}

/^\*/ {
	body = body "<ul>\n"

	while (match($0, /^\*[ ]*/) != 0) {
		gsub("&", "\\&amp;")
		gsub("<", "\\&lt;")
		gsub(">", "\\&gt;")
		gsub("'", "\\&apos;")
		gsub("\"", "\\&quot;")

		body = body sprintf("<li>%s</li>\n", substr($0, RSTART + RLENGTH))

		if (getline != 1) break
	}

	body = body "</ul>\n"
	next
}


{ 
	gsub("&", "\\&amp;")
	gsub("<", "\\&lt;")
	gsub(">", "\\&gt;")
	gsub("'", "\\&apos;")
	gsub("\"", "\\&quot;")

	body = body sprintf("<p>%s</p>\n", $0)
}

END {
	delete subHeadings[0] # this index at subHeadings contains nothing

	if (length(subHeadings) > 0) {
		tocText = "<details id='toc'> <summary>Table of content</summary> <ol>\n"

		for (i in subHeadings) {
			headingTextAsID = tolower(subHeadings[i])
			gsub(/^[ ]*/, "", headingTextAsID)
			gsub(/[ ]*$/, "", headingTextAsID)
			gsub(" ", "_", headingTextAsID)

			tocText = tocText sprintf("<li><a href='#%s'>%s</a></li>\n", headingTextAsID, subHeadings[i])
		}

		tocText = tocText "</ol> </details>\n"
	}

	navbarText = genHTMLNavbar(FILENAME, sourcePath, destPath)

	dirname = getDirname(FILENAME)

	stylesheetPath = getRelativePath(dirname, sourcePath "/style.css")

	printf "<!DOCTYPE html>\n\
<html>\n\
	<head>\n\
		<title>%s</title>\n\
		<meta name='author' content='techn0path' />\n\
		<meta name='referrer' content='no-referrer' />\n\
		<meta name='viewpoint' content='width=device-width,initial-scale=1.0' />\n\
\n\
		<link rel='stylesheet' href='%s'/>\n\
	</head>\n\
	<body>\n\
		<header>\n\
%s\n\
		</header>\n\
		<article id='content'>\n\
		%s\n\
		%s\n\
		<p>If you have any suggestions, let me know at <a href='mailto:logicartist123@gmail.com'>logicartist123@gmail.com</a>. Sorry, I still use GMail. :/</p>\n\
		</article>\n\
	</body>\n\
</html>", \
	title, \
	stylesheetPath, \
	navbarText, \
	tocText, \
	body
}
